/* memo.
	名前のすみわけ（理想であって、実際はそうなっていないかもしれない）
		"イベント"
			イベントドリブンのイベント		イベント	という
			シナリオスクリプトのイベント	ストーリー	という
			
		"ストーリー"
			以下のような、その場所でどんな行動をとったらこの関数、という構造のオブジェクトをstoryと呼ぶ。
				{
					"walk" : function() {...},
					"outer.goton" : function() {...},
				}
			storyの配列をstoriesと呼ぶ。
			storyを構成する story.walk や story[ "outer,goton" ] をshortStoryと呼ぶ。
				
		"アイテム"
			もちもの		トレジャー
			メニュー項目	アイテム
			
		キャラの状態
			直近の状態		status		毒を受けたとか
			恒常的な状態		property	力がどれだけあるとか
		
		座標
			グラフィクス		gx, gy, gw, gh
			マス目のある画面	col, row, cols, rows
			データ上の位置		x, y, w, h
		マス目									１マスの名称	座標
			画面のマス目							tile		style.gridCol,style.gridRow
			要素内に作成したマス目		inner		cell		col,row
		
		配列変数の名前と、同じデータを持つ連想配列変数の名前
			配列		-s	たとえばcharacters
			連想配列	-z	たとえばcharacterz

			そのとき、たとえばsprites、spritezの読み方について
			配列時		sprites	スプライトス
			連想配列時	spritez	スプライトス
			※どちらも-スと読む。つまり-zは見た目の区別でしかない
			
			また、childの複数形childrenについて -z を表現できないので、
			childs, childz とする。
		連想配列のkeyとキー入力のkey
			連想配列	name
			キー入力	key
	言葉の意味
		tweak		調整する
		strategy	広い視野で物事を見て全体の方向性を決めていく
		tactic		実務レベルでどのように対処するのかという部分的な一つの計画

	async関数とawait
		
		awaitはasync関数を暗黙的に分割します。
		
		async test() {
			処理１
			await 関数など
			処理２
		}
		処理１までは通常の関数と同じように動作し、
		awaitの後の処理２はawaitの行が終わる（= Promiseの解決が履行される）までは実行されません。
		また、処理２が終わった後は、test関数が呼び出しされた場所には戻りません。
		test関数の最初のawaitの時点で、プログラムは２つ（並列）に枝分かれします。
		片方はtest()を出てtest()を呼び出した次の行へ進み、もう片方はawait以降を実行します。
		await以降を実行した後は、どこにも戻らず、終了します。
		このRPGのプログラムにおいては、枝分かれすることはあまり価値がありません。
		１つの関数を途中で止めることができるawaitに価値があります。
*/

// |
// V
//---■■■ 1  Utl ■■■
class Utl {//c
	static arrayForward( array, element ) {//m
		//配列内の要素を末尾に置く
		
		let idx = array.indexOf( element );
			//check.
			if( idx == -1 ) {
				alert( "Utl.arrayForward() 失敗した" );
				return;
			}
			
		array.splice( idx, 1 );
		array.push( element );
	}
	static accessBy( /**/ ) {//m
		//test.test2.test3のようなアクセスで、test2がnullだとエラーで止まる。
		//accessBy( test, "test2", "test3" )とすれば止まらず、nullを返す。
		//try,catchもあるが、なじみがないので。
		let array = Array.from( arguments );
		let object = array.shift();
		do {
			if( object === null ) return null;
			if( object === undefined ) return undefined;
			object = object[ array.shift() ];
		} while( array.length );
		return object;
	}
	static array2object( array, object, keyMaker ) {//m
		//配列を連想配列に変換する。その際の名前はkeyMakerが作る
		//objectは作成済みの場合
		for( let element of array ) {
			object[ keyMaker( element ) ] = element;
		}
	}

	static objectFrom( array, keyMaker ) {//m
		//配列を連想配列に変換する。その際の名前はkeyMakerが作る
		//objectは作成済みではない場合
		let object = new Object();
		for( let element of array ) {
			object[ keyMaker( element ) ] = element;
		}
		return object;
	}
	static lengthBin( str ) {//m
		//漢字を２バイト、半角を１バイトとしてバイト数を計算　（UTF8を前提）
		return encodeURI( str ).replace( /%20/, "*" ).replace( /%..%..%../g, "**" ).length;
	}
	static lengthBinDiv2( str ) {//m
		//lengthBinの結果を２で割り、小数点切り上げ
		return Math.ceil( Utl.lengthBin( str ) / 2 );
	}
	static maxLength( strArray ) {//m
		//文字列配列中で、一番長い文字列の文字数を返す
		let v = 0;
		for( let str of strArray ) v = Math.max( v, String( str ).length );
		return v;
	}
	static maxLengthBin( strArray ) {//m
		//そのバイト版
		let v = 0;
		for( let str of strArray ) {
			v = Math.max( v, Utl.lengthBin( String( str ) ) );
		}
		return v;
	}
	static maxLengthBinDiv2( strArray ) {//m
		//その、２で割り、小数点切り上げ版
		return Math.ceil( Utl.maxLengthBin( strArray ) / 2 );
	}
	static copyObjectKai( object, history ) {//m
		//オブジェクトを厳密にコピーするの改造版
			//check.　最初の呼び出し
			if( history == undefined ) {
				history = {
					origin : new Array(),
					copy : new Array(),
				}
			}
		if( object instanceof Object ) {
				//check.　すでにコピーしたことがある（無限ループ検知）
				if( history.origin.includes( object ) ) {
					let idx = history.origin.indexOf( object );
					return history.copy[ idx ];
				}
			history.origin.push( object );
			let copy;
				if( object instanceof Array ) {
					copy = new Array();
					history.copy.push( copy );
					for( let i = 0; i < object.length; i++ ) {
						copy[ i ] = Utl.copyObjectKai( object[ i ], history );
					}
				} else {
					copy = new Object();
					history.copy.push( copy );
					for( let name in object ) {
						//改造箇所はここだけ
						//オブジェクトのメンバでcopy:trueを持つ場合に限り
						//厳密にコピーをする。
						//copy:trueがないと、コピー元のオブジェクトの参照になる。
						//モンスターやトレジャーはカタログをコピーするが
						//HPや使用回数だけはカタログへの参照ではなくコピーにしたい。
						if( object[ name ].copy ) {
							//HPや使用回数はこちら
							copy[ name ] = Utl.copyObjectKai( object[ name ], history );
						} else {
							//その他定義は読むだけなので参照でよい
							copy[ name ] = object[ name ];
						}
					}
				}
			return copy;
		} else {
			return object;
		}
	}
}//Utl
//---■■■ 2 Element ■■■
class Element {//c
	constructor( style, app ) {
		this.app = app ? app : this;
			//check.
			if( this == app && style.gridCellSize == undefined )
				alert( "appに限り、styleにgridCellSizeの定義が必要です。" );
			
		//デフォルト値と上書き
		this.style = {
			gridCol : 0,
			gridRow : 0,
			gridCols : 5,
			gridRows : 5,
		};
		this.assignStyle( style );
		
		this.name = this.constructor.name;
		this.childs = new Array();
		this.prevFocus = null;
	}
	assignStyle( newStyle ) {
		Object.assign( this.style, newStyle );
		this.x = this.style.gridCol * this.app.style.gridCellSize;
		this.y = this.style.gridRow * this.app.style.gridCellSize;
		this.w = this.style.gridCols * this.app.style.gridCellSize;
		this.h = this.style.gridRows * this.app.style.gridCellSize;
	}
	typeKey( key ) {//m
	}
	senseKey( key ) {//m
	}
	draw( cc ) {//m
		this.drawWindow( cc );
		this.drawChildren( cc );
	}
	drawWindow( cc ) {//m
	
		//ウィンドウ
		let x = - this.app.style.gridCellSize / 2;
		let y = - this.app.style.gridCellSize / 2;
		let w = this.w + this.app.style.gridCellSize;
		let h = this.h + this.app.style.gridCellSize;
		
		cc.fillStyle = "black";
		cc.fillRect( x, y, w, h );

		//ウィンドウ　面
		cc.fillStyle = "white";
		cc.fillRect( x, y, w, h );
		
		//ウィンドウ　枠
		if( this.app.currentElement == this ) {
			cc.strokeStyle = "yellow";
			cc.strokeRect( x + 1, y + 1, w - 2, h - 2 );
		}
		cc.strokeStyle = "black";
		cc.strokeRect( x, y, w, h );
	}
	drawChildren( cc ) {//m
		for( let child of this.childs ) {
			cc.save();
				cc.translate( child.x, child.y );
				child.draw( cc );
			cc.restore();
		}
	}
	searchStory() {//m
	}
	focus() {//m
			//check.　すでにフォーカスされている。
			if( this.app.currentElement == this ) {
				console.log( "already", this.name, "focused" );
				return;
			}

		if( this.parent )
			console.log( "focus", this.name, "at", this.parent.name );
		else
			console.log( "focus", this.name );
		
		if( this.prevFocus == undefined )
			this.prevFocus = this.app.currentElement;
		this.app.currentElement = this;
		
		//前面に移動
		if( this.parent ) {
			let idx = this.parent.childs.indexOf( this );
			this.parent.childs.splice( idx, 1 );
			this.parent.childs.push( this );
		}
	}
	appendChild( child, nofocus ) {//m
		this.childs.push( child );
		child.setParent( this );
		if( ! nofocus ) child.focus();
		return child;
	}
	appendFloat( child ) {
		this.appendChild( child, true );
	}
	setParent( parent ) {
		this.parent = parent;
			//check.
			for( let name in this.style ) {
				if( this.style[ name ] == "inherit" ) {
					this.style[ name ] = parent.style[ name ];
				}
			}
	}
	removeChild( child ) {//m
		console.log( "remove", child.name, "from", this.name );
		let idx;
		
		if( ( idx = this.childs.indexOf( child ) ) > -1 ) {
			this.childs.splice( idx, 1 );
			if( child.prevFocus ) child.prevFocus.focus();
		} else {
			alert( "警告：正体不明のオブジェクトをelement.removeChild()しようとしました." );
		}

		//メッセージウィンドウの場合はthis.appからも削除
		if( child == this.app.messageWindow ) {
			delete this.app.messageWindow;
		}

		return child;
	}
	removeAllElements() {//m
		for( let child of this.childs ) {
			child.removeAllElements();
		}
		this.parent.removeChild( this );
	}

}//Element
// |
// V
//---▼▼▼ 3 App ▼▼▼
class App extends Element {//c
	constructor( canvasId ) {

		let canvas = document.getElementById( canvasId );
		let cc = canvas.getContext( "2d", { alpha : false } );
		
		let logiczoom = 1;
		let canvasWidth = 256 / logiczoom;	//プログラム上の論理的サイズ
		let canvasHeight = 224 / logiczoom;
		canvas.width = canvasWidth;
		canvas.height = canvasHeight;
		if( 0 ) canvas.style.imageRendering = "crisp-edges";	//アンチエイリアス
		
		/*	解像度変更機能：
		
			mozaic	canvasWidth	canvas.width	scale	
			0.5		256			2048			8		さらにきめ細かい
			1		256			1024			4		きめ細かい（標準）
			2		256			512				2		少し荒い
			4		256			256				1		ファミコン風
		*/
		if( 1 ) {
			let mozaic = 1;									//見た目の粒の大きさ
			canvas.width = canvasWidth / mozaic * 4;		//物理的
			canvas.height = canvasHeight / mozaic * 4;
			cc.scale( 4 / mozaic, 4 / mozaic );	//物理１ピクセル当たりの論理ピクセル数
		}
		
		let gridCellSize = canvasWidth / 32;
		
		super( {
			gridsPerInnerCell : 2,	//内部１セルあたり、何gridか。
				//gridはアプリ内での絶対的なマス目。
				//例えば、gridが８ピクセルで、マップチップが16ピクセルなら、チップ当たり2grid
				//という意味。
			gridCol : 0,
			gridRow : 0,
			gridCols : Math.floor( canvasWidth / gridCellSize ),
			gridRows : Math.floor( canvasHeight / gridCellSize ),

			gridCellSize : gridCellSize,		//全画面共通の最小単位。
			innerCellSize : gridCellSize * 2,	//その画面の単位。マップチップ等
		}, null );

		this.cc = cc;
		this.cc.canvasWidth = canvasWidth;
		this.cc.canvasHeight = canvasHeight;
	}
	// |
	// V
	async initialize() {//m
	
				
		//---メトロノーム
		this.metronomez = new Object();
			this.addMetronome( "t1000_i2" );
			this.addMetronome( "t500_i2" );
			this.addMetronome( "t250_i2" );
			//カーソル専用（手動追加）
			this.metronomez.forCursor = {
				maxTime : 500,
				maxIndex : 0,
				time : 0,
				index : 0,
				toggle : true,
				changed : false,
			}
/*
				メトロノームの各フラグの変化タイミング図解（タイミングチャート）
				maxTime : 250,
				maxIndex : 3,　の場合の各フラグの動き↓

				（経過時間）		0	...	250	...	500	...	750	...	1000 ...(ms)
				
				time : 0,			0 ..249 0 ..249	0 ..249	0 ..249	0 .. ...(ms)
				
				index : 0,			000000001111111122222222000000001111 ...
				
				toggle : true,		~~~~~~~~________~~~~~~~~________~~~~ ...

				changed : false,	________~_______~_______~_______~___ ...
									
									~ : true
									_ : false
									... : 略
									※このグラフ中の１文字単位の縦１列はrequestAnimationFrameに相当する
									
				つまり、requestAnimationFrameごとに、
				timeは経過時間を増やしていきmaxTimeになると0に戻る
				indexはtimeがmaxTimeになるごとに１増え、maxIndexになると0に戻る
				toggleはtimeがmaxTimeになるごとに真偽を反転する
				changedはtimeがmaxTimeになったときだけ真になる
				
				maxTimeでアニメのパラパラのスピード決め
				toggleやchangedでアニメのパラパラの実行を判断
				indexでアニメのパラパラの画の切り替え
*/

		this.anms = new Array();

		this.spritez = {
			player : {
				images : [ "←", "－", "↑", "｜", "→", "－", "↓", "｜" ],
				metronome : this.metronomez.t500_i2,
			},
		}
		
		this.artz = {
			spritez : this.spritez,
			bgms : new Object(),
			soundz : new Object(),
			imagez : new Object(),
		}
		//---素材のプリロード
		let dir = location.pathname.replace( /%20/g, " " ).replace( /\/[a-z0-9_ .-]*$/i, "" ) + "/";
		
			this.cc.fillStyle = "cyan";
			let str = "want a moment"
			this.cc.fillText( str, ( this.cc.canvasWidth - this.cc.measureText( str ).width ) / 2, 100 );

		await Promise.all( [
			this.load( "bgm", "field", dir + "_bgm/街/MusMus-CT-NV-24/MusMus-CT-NV-24.mp3" ),
			this.load( "bgm", "town", dir + "_bgm/街/MusMus-BGM-078/MusMus-BGM-078.mp3" ),
			this.load( "sound", "pushA", dir + "_sound/pushA.mp3" ),
			this.load( "sound", "pushWall", dir + "_sound/pushWall.mp3" ),
			this.load( "sound", "walk", dir + "_sound/walk.wav" ),
			this.load( "sound", "damage", dir + "_sound/musmus_other_set/othr10.mp3" ),
			this.load( "image", "slime", dir + "_img/slime.png" ),
			this.load( "image", "battleBG", dir + "_img/battle.png" ),
		] );
		console.log( "素材のロード完了." );

		this.monsters = new Array();	//戦闘中のモンスター
		this.characters = new Array();
		this.characterz = new Object();
		this.treasureCatalog = new Object();
		
		//---■メニュー
		this.menusrcs = [
			{
				name : "yesno",
				title : "",
				items : [
					{
						name : "はい",
					},
					{
						name : "いいえ",
					}
				],
			},
			{
				name : "camp",
				title : "コマンド？",
				argzName : "cmd",
				items : [
					{
						name : "どうぐ",
						title : "だれの？",
						items : this.characters,
						argzName : "char1",
						assistMenu : {
							title : "どれ？",
							argzName : "dougu",
							row : 1,	//だれでも表示高さを固定して表示
						},
					},
					{
						name : "しらべる",
					},
					{
						name : "まほう",
						title : "だれの？",
						items : this.characters,
						argzName : "char1",
					},
				],
			},
			//---
			{
				name : "つかう",
				title : "だれに？",
				items : this.characters,
				argzName : "char2",
				story : async function( argz ) {
					this.openMessage();
					await this.writeMessage( argz.char1.name + "は" + argz.char2.name + "に" + argz.dougu.name + "を使った！", { wait : true } );
					
					let lostFlg = true;
					if( argz.dougu.property.limit != undefined ) {
						//limitが０になったらなくなるタイプ
						argz.dougu.property.limit --;
							//check.
							lostFlg = argz.dougu.property.limit == 0;
							if( lostFlg ) {
								await this.writeMessage( "なんと　" + argz.dougu.name + "　は壊れてしまった！", { clear : true, wait : true } );
							}
					}
					if( lostFlg ) {
						let idx = argz.char1[ argz.cmd.name ].indexOf( argz.dougu );
						argz.char1[ argz.cmd.name ].splice( idx, 1 );
					}
					this.closeMessage();
				},
			},
			{
				name : "わたす",
				title : "だれに？",
				items : this.characters,
				argzName : "char2",
				story : async function( cmd, char1, dougu, cmd2, char2 ) {
					this.openMessage();
					await this.writeMessage( char1.name + "は" + char2.name + "に" + dougu.name + "を渡した！", { wait : true } );
					this.closeMessage();
				},
			},
			{
				name : "すてる",
				story : async function( cmd, char1, dougu, cmd2 ) {
					this.openMessage();
					await this.writeMessage( char1.name + "は" + dougu.name + "を捨てた！", { wait : true } );
					this.closeMessage();
				},
			},
		]
		this.menusrcz = Utl.objectFrom( this.menusrcs, element => element.name );
		//---■トレジャー
		this.treasureCatalog = {
			"どうけん" : {
				
			},
			"せいどうけん" : {
				
			},
			"てっけん" : {
				
			},
			"こうてつけん" : {
				
			},
			"ひほう" : {
				property : {
					copy : true,
					limit : 3,
				},
				assistMenuCamp : {
					title : "どうする？",
					argzName : "dousuru",
					items : [
						this.menusrcz[ "つかう" ],
						this.menusrcz[ "わたす" ],
						this.menusrcz[ "すてる" ],
					],//items
				},
			},
			"なんこう" : {
				property : {
					copy : true,
				},
				assistMenuCamp : {
					title : "どうする？",
					argzName : "dousuru",
					items : [
						this.menusrcz[ "つかう" ],
						this.menusrcz[ "わたす" ],
						this.menusrcz[ "すてる" ],
					],//items
				},//nextMenu
				assistMenuBattle : this.menusrcz[ "つかう" ],
			},//なんこう
		}//treasureCatalog
		
			//tweak.　キーを名前として要素内に登録
			for( let name in this.treasureCatalog ) {
				this.treasureCatalog[ name ].name = name;
			}
		
		//---■戦闘思考ルーチン
		this.tacticz = {
			slime : function( ourselves, enemies, battleLog ) {
				/*
				result = {
					menus
					argz
					story
					selectedItem
				}
				*/
				return {
					argz : {
						enemy : enemies[ 0 ],
					},
					story : this.battleStories[ "たたかう" ],
				}
			},
		}
		//---■モンスター
		this.monsterCatalog = {
			"モモスラ" : {
				image : this.artz.imagez.slime,
				property : {
					copy : true,
					hp : 10,
					atack : 5,
					defence : 5,
				},
				tactic : this.tacticz.slime,
			}
		}
			//tweak.　キーを名前として要素内に登録
			for( let name in this.monsterCatalog ) {
				this.monsterCatalog[ name ].name = name;
			}
		
		//---■戦闘コマンド　実行
		
		this.battleStories = {
			"たたかう" : async function( argz ) {
				await this.writeMessage( argz.self.name + "は " + argz.enemy.name + " にこうげき！\n", { clear : true, } );
					//check.
					if( argz.enemy.property.hp == 0 ) {
						await this.writeMessage( "しかし" + argz.enemy.name + "はすでに倒れている！", { wait : true, } );
						return;
					}
				let enemyDamage = argz.self.property.atack;
					enemyDamage -= argz.enemy.property.defence;
				
				if( enemyDamage > 0 ) {
					await this.effect_imageBure();
					await this.writeMessage( argz.enemy.name + "に " + enemyDamage + " ポイントのダメージをあたえた！", { wait : true, } );
					argz.enemy.property.hp -= enemyDamage;
						//check.
						if( argz.enemy.property.hp <= 0 ) {
							argz.enemy.property.hp = 0;
							await this.writeMessage( "\n" + argz.enemy.name + "は倒れた！", { wait : true, } );
						}
				} else {
					await this.writeMessage( "ミス！ダメージをあたえられない！", { wait : true, } );					
				}
			}
		}
		
		//---■戦闘コマンド
		this.batcomz = {
			"たたかう" : {
				title : "あいては？",
				items : function() {
					return this.monsters.filter( m => m.property.hp > 0 );
				},
				argzName : "enemy",
				row : 1,
				story : this.battleStories[ "たたかう" ],
			},
		};
			//tweak.　キーを名前として要素内に登録
			for( let key in this.batcomz ) {
				this.batcomz[ key ].name = key;
			}

		//---■キャラクター

		this.characters.push( {
			name : "キャラ１",
			property : {
				hp : 20,
				atack : 10,
				defence : 10,
			},
			"どうぐ" : [
				{ name : "つるぎ" },
				this.treasureCatalog[ "なんこう" ],
				Utl.copyObjectKai( this.treasureCatalog[ "ひほう" ] ),
				{ name : "つるぎ" },
			],
			batcoms : [
				this.batcomz[ "たたかう" ],
			],
		} );
		
		this.characters.push( {
			name : "キャラ２",
			property : {
				atack : 10,
				defence : 10,
			},
			"どうぐ" : [
				{ name : "なんこう２" },
				{ name : "つるぎ" },
			],
			batcoms : [
				this.batcomz[ "たたかう" ],
			],
		} );
		this.characters.push( {
			name : "キャラ３",
			property : {
				atack : 10,
				defence : 10,
			},
			"どうぐ" : [
				{ name : "なんこう３" },
				{ name : "つるぎ" },
			],
			batcoms : [
				this.batcomz[ "たたかう" ],
			],
		} );
		Utl.array2object( this.characters, this.characterz, element => element.name );

	}//initialize()
	// |
	// V
	//プリロード
	load( type, name, src ) {//m
		return new Promise( function( promiseOk ) {
			let object;
			console.log( type, name, "loading.." );
			switch( type ) {
				case "image":
					object = new Image();
					this.artz.imagez[ name ] = object;
					object.onload = function() {
						console.log( "\t", type, name, "loaded." );
						promiseOk();
					}
					object.src = src;
					break;
				case "bgm":
					object = new Audio();
						object.oncanplaythrough = function() {
							this.artz.bgms[ name ] = object;
							object.oncanplaythrough = null;
							console.log( "\t", type, name, "loaded." );
							promiseOk();
						}.bind( this );
						object.src = src;
						object.load();
					break;
				case "sound":
					object = new Audio();
						object.oncanplaythrough = function() {
							this.artz.soundz[ name ] = object;
							object.oncanplaythrough = null;
							console.log( "\t", type, name, "loaded." );
							object.rapidPlay = function() {
								object.pause();
								object.currentTime = 0;
								object.play();
									/*
										同じ音楽ファイルの連続演奏時は
										最初にリセットをかける必要がある。
										rapid＝急速
									*/
							}
							promiseOk();
						}.bind( this );
						object.src = src;
						object.load();
					break;
			}
		}.bind( this ) );
	}//load()

	onkeydownx( e ) {//m

			//debug.　現在の階層構造を表示
			if( e.which == 79 ) {	//'o'キー
				console.log( "=== 階層構造 ===" );
				console.log( this.debug_dir( this ) );
				console.log( "=== /階層構造 ===" );
				return;
			}

		if( this.keys.indexOf( e.which ) == -1 ) {
			this.keys.push( e.which );	//キーセンス　オン
			
			this.currentElement.typeKey( e.which );
				//キータイプ　実行
		}
	}
	// |
	// V
	onkeyupx( e ) {//m
		let idx = this.keys.indexOf( e.which );
		if( idx > -1 ) this.keys[ idx ] *= -1;
			//キーセンス　オフ要求（同値のマイナスで示す）
	}

	addMetronome( name /*other args*/ ) {//m
		//オーバーロード（メソッド名が同じで引数のパターンが異なる）の模造
		//”関数の先頭で引数の型を判定する条件分岐で対応”
		let argsId = "";
			for( let arg of Array.from( arguments ) ) {
				argsId += "," + ( typeof arg ).substr( 0, 1 );
			}
		let metronome;
			switch( argsId ) {
				case ",s" :		//フォーマット名称　t時間_iインデックス上限
					let tokens = name.split( /_/ );
					metronome = {
						maxTime : Number( tokens[ 0 ].substr( 1 ) ),
						maxIndex : Number( tokens[ 1 ].substr( 1 ) ),
						time : 0,
						index : 0,
						toggle : false,
						changed : false,
					}
					this.metronomez[ name ] = metronome;
					break;
				case ",s,n" :	//自由な名称、時間
					let tm = arguments[ 1 ];
					metronome = {
						maxTime : tm,
						maxIndex : 0,
						time : 0,
						index : 0,
						toggle : false,
						changed : false,
					}
					this.metronomez[ name ] = metronome;
					break;
				default:
					alert( "addMetronome()にて、未定義の引数リスト:[" + args + "]" );
			}
		return metronome;

	}//addMetronome()
	// |
	// v
	removeMetronome( metronome ) {//m
		delete this.metronomez[ metronome.name ];
	}

	start() {//m
		//------------

		this.focus();
		this.keys = new Array();
		
		this.memoryCard = {
			player : {
				position : {
					map : App.worldmap,
					x : 0,
					y : 0,
				},
				direction : 0,
				gold : 200,
			},
		}

		//タイトル画面から始める
		let titleScreen = new TitleScreen( {
			innerCellSize : this.style.gridCellSize * 2,
			gridCol : 0,
			gridRow : 0,
			gridCols : this.style.gridCols,
			gridRows : this.style.gridRows,
		}, this );
		this.appendChild( titleScreen );
		

		this.startFrame();

	}//start()
	// |
	// V
	stop() {//m
	}
	// |
	// V
	startFrame() {//m
		this.keys.length = 0;
		this.beforeTime = 0;
		this.frame( 0 );
		onkeydown = this.onkeydownx.bind( this );
		onkeyup = this.onkeyupx.bind( this );
	}
	// |
	// V
	stopFrame() {//m
		cancelAnimationFrame( this.timerId );
		this.keys.length = 0;
		onkeydown = null;
		onkeyup = null;
	}
	// |
	// V
	frame( tm ) {//m
		//経過時間
		let diff = tm - this.beforeTime;
		this.beforeTime = tm;

		//メトロノーム更新
		for( let metronome of Object.values( this.metronomez ) ) {
			metronome.time += diff
			
			//check.　そのメトロノームは時を刻んだ
			if( metronome.time >= metronome.maxTime ) {
				metronome.time = 0;
				metronome.toggle = ! metronome.toggle;
				metronome.changed = true;
				metronome.index ++;
					//check.　インデックスリセット
					if( metronome.index == metronome.maxIndex ) metronome.index = 0;
			} else {
				metronome.changed = false;
			}
		}

		//キーセンス　処理
		for( let i = this.keys.length - 1; i >= 0; i-- ) {
			let key = this.keys[ i ];
			this.currentElement.senseKey( Math.abs( key ) );
				//check.　キーセンス　オフ
				if( key < 0 ) this.keys.splice( i, 1 );
		}

		//1ドットスクロール実行
		if( this.mapScreen )
		if( this.mapScreen.scroll.isAnimating ) {
			this.mapScreen.frameForScroll();
		}
		
		this.storyCheck();
		
		//アニメ処理　（forの途中で要素削除するので逆順）
		for( let i = this.anms.length - 1; i >= 0; i-- ) {
			if( this.anms[ i ].call( this ) ) this.anms.splice( i, 1 );
		}

		this.draw( this.cc );
		
		this.timerId = requestAnimationFrame( this.frame.bind( this ) );
	}
	// |
	// V
	async storyCheck() {//m
		//ストーリー検索＆実行
		let stories = this.currentElement.searchStory();
			//check.
			if( ! stories ) return;
		for( let story of stories ) {
			let res = await story.story.call( this, story.action );
				//check.　真を返したら以降を打ち切り（町に入るところで戦闘発生等防ぐため）
				if( res ) break;
		}
	}
	
	openMessage( style ) {//m
//		this.artz.soundz.pushA.rapidPlay();
		
			//check.　デフォルト値と上書き
			style = Object.assign( {
				gridCol : 2,
				gridRow : 18,
				gridCols : 28,
				gridRows : 8,
				innerCellSize : this.style.innerCellSize,
			}, style );
			//check.　メッセージウィンドウがすでにあるときは一度閉じる
			if( this.messageWindow ) {
				this.closeMessage();
			}
		this.messageWindow = new MessageWindow( style, this.app );
		this.appendChild( this.messageWindow );
	}
	// |
	// V
	closeMessage() {//m
		this.messageWindow.close();
	}
	// |
	// V
	async writeMessage( message, flgs ) {//m
			//check.
			if( ! flgs ) flgs = {};
			//check.
			if( flgs.clear ) this.messageWindow.clear();
			if( flgs.wait ) this.messageWindow.focus();

		await this.messageWindow.write( ...arguments );
 	}

	yesno( closeFlg ) {//m
		this.artz.soundz.pushA.rapidPlay();
		
		let yesnoMenu = new Menu( "selectmode", "", this.menusrcz.yesno, {
			gridCol : 26,
			gridRow : 25,
		}, this );
		this.appendChild( yesnoMenu );

		return new Promise( function( promiseOk ) {
			console.log( "promised 'yesno'" );
			yesnoMenu.promiseOk = promiseOk;
		}.bind( this ) ).then( function( result ) {
			console.log( "promise 'yesno' completed" );
			this.removeChild( yesnoMenu );
			if( Utl.accessBy( result, "selectedItem" ) ) {
				return result.selectedItem.name == "はい";
			} else 
				return false;
		}.bind( this ) );
	}
	// |
	// V
	//---●●● shop ●●●
	async shop( menusrc, wordingz ) {//m
		let treasureMenu = new Menu( "selectmode", "", menusrc, {
			gridCol : 2,
			gridRow : 2,
		}, this );
		this.mapScreen.appendChild( treasureMenu );

		//所持金表示
		let stat = new List( {
			columns : [ { key : "title" }, { key : "value", align : "right" } ],
			items : [
				{
					title : "エン",
					value : function() { return this.memoryCard.player.gold; },
				},
			],

		}, {
			gridCol : treasureMenu.style.gridCols + 2,
			gridRow : 0,
		}, this.app );
		treasureMenu.appendFloat( stat );
		
		this.openMessage();
		
		while( 1 ) {
			//商品を選ぶ
			await this.writeMessage( wordingz[ "何を買うんだ？" ] + "\n" );
			treasureMenu.focus();
			let result = await treasureMenu.select();
			
				//check.　お店から退出
				if( ! result ) break;
				
			let selectedItem = result.selectedItem;
			
			this.messageWindow.focus();	//←見た目のため
			await this.writeMessage( selectedItem.name + wordingz[ "だな。" ] + "\n" );
			await this.writeMessage( selectedItem.price + "エン" + wordingz[ "だ。買うかね？" ] + "\n" );
			if( await this.yesno() ) {
				this.memoryCard.player.gold -= selectedItem.price;
				await this.writeMessage( wordingz[ "まいどあり！" ] + "\n", { wait : true } );
				await this.writeMessage( "\n" );
			}
		}
		await this.writeMessage( wordingz[ "ありがとうございました！" ], { wait : true } );
		this.closeMessage();
		this.mapScreen.removeChild( treasureMenu );
	}

	encounter() {//m
		this.battle();
	}
	// |
	// V
	//---●●● battle ●●●
	async battle() {//m
		this.monsters = new Array();
		let max = Math.floor( Math.random() * 5 ) + 1;
		console.log( max );
		for( let i = 0; i < max; i++ ) {
			this.monsters[ i ] = Utl.copyObjectKai( this.monsterCatalog[ "モモスラ" ] ),
			this.monsters[ i ].name += i + 1;
		}
		let masterOfMonsters = null;
		
		let battleLog = new Array();
		
		let battleView = new BattleView( this.monsters, {
			gridCol : 6,
			gridRow : 6,
			gridCols : 20,
			gridRows : 12,
		}, this );
		
		await this.effect_battleStart( battleView );

		this.mapScreen.appendFloat( battleView );
		
		this.openMessage();
		let m = this.monsters.map( monster => monster.name ).join( "、" );
		await this.writeMessage( m + "があらわれた！", { wait : 1 } );
		this.closeMessage();

		let theEndOfTheBattle;
		do {
			let commandz = new Object();

				this.openMessage( {
					gridCol : 9,
					gridRow : 18,
					gridCols : 21,
					gridRows : 8,
				} );
					
				//プレイヤー側のコマンド決定
				for( let i = 0; i < this.characters.length; i++ ) {
					let character = this.characters[ i ];
						//check.
						if( character.property.hp == 0 ) continue;
						
					let cmdMenu = new Menu( "selectmode", "", {
						name : "batcom",
						title : character.name,
						items : character.batcoms,
					}, {
						gridCol : 1,
						gridRow : 18,
					}, this.app );
					this.appendChild( cmdMenu );
					
					this.writeMessage( character.name + "はどうする？", { rapid : true, clear : true } );
					
					let command = await cmdMenu.select();
					cmdMenu.removeAllElements();
						//check.
						if( ! command ) {
							i -= i == 0 ? 1 : 2;
							continue;
						}
						//tweak.
						command.argz.self = character;
						
					commandz[ character.name ] = command;
				}//for character
				this.closeMessage();

				//モンスター側のコマンド決定
				if( masterOfMonsters ) {
					//リーダーが思考する場合　未使用　未デバッグ
					let resultz = masterOfMonsters.strategy.call( this, this.monsters, this.characters, battleLog );
					Object.assign( commandz, resultz );
				} else {
					//それぞれが思考する場合
					for( let monster of this.monsters ) {
							//check.
							if( monster.property.hp == 0 ) continue;
						commandz[ monster.name ] = monster.tactic.call( this, this.monsters, this.characters, battleLog );
						commandz[ monster.name ].argz.self = monster;
					}
				}
			
			this.openMessage();
			for( let name in commandz ) {
					//check.
					if( commandz[ name ].argz.self.property.hp == 0 ) continue;
				let command = commandz[ name ];
				
				await command.story.call( this, command.argz );
				
					//check.　味方の生存者いない
					if( ! this.characters.some( c => c.property.hp > 0 ) ) {
						await this.writeMessage( "\n" + characters[ 0 ].name + "たちは全滅した…", { wait : true } );
						theEndOfTheBattle = true;
						break;
					} else if( ! this.monsters.some( m => m.property.hp > 0 ) ) {
						//敵の生存者いない
						await this.writeMessage( "\nモンスターたちを倒した！", { wait : true } );
						theEndOfTheBattle = true;
						break;
					} else {
						theEndOfTheBattle = false;
					}
			}
			this.closeMessage();
			
		} while( ! theEndOfTheBattle );
		
		this.mapScreen.removeChild( battleView );
	}
	async delay( ms ) {
		await new Promise( promiseOk => setTimeout( promiseOk, ms ) );
	}
	async effect_battleStart( element ) {//m
		await this.effect_flash( "red" );
		await this.effect_flash( "yellow" );
		await this.effect_flash( "red" );
		this.stopFrame();
		
		let cc = this.cc;
		let step = 13;
		let movl = 200
		for( let i = 1; i <= step; i ++ ) {
			let theta = Math.PI * 2 / step * i;
			let alpha = 1 / step * i;
			let movx = movl - movl / step * i;
			this.draw( cc );
			cc.save();
			cc.globalAlpha = alpha;
			cc.translate( element.x + movx, element.y );
			cc.rotate( theta );
			
			element.draw( cc );
			cc.restore();

			await this.delay( 60 );
		}
		
		this.startFrame();
	}
	effect_imageBure() {//m
		
	}
	effect_flash( color ) {//m
		let effect = new Element( {}, this );
			//描画
			effect.draw = function( cc ) {
				cc.save();
					cc.globalCompositeOperation = "difference";
					cc.fillStyle = color;
					cc.fillRect( 0, 0, cc.canvasWidth, cc.canvasHeight );
				cc.restore();
			}

		//エフェクト開始

		let cnt = 0;
		this.appendChild( effect );
		
		//音声
		this.artz.soundz.damage.rapidPlay();

		return new Promise( function( promiseOk ) {
			this.anms.push( function(a) {
				cnt++;
					//check.
					if( cnt == 10 ) {	//何フレーム上記drawを行うか
						//エフェクト終了
						this.removeChild( effect );
						promiseOk();
						return true;
					}
			} );
		}.bind( this ) );
	}
	debug_dir( object, tab, idx ) {//m
			//check.
			if( tab == undefined ) tab = "";
		let str = tab;
			if( object != this ) {
				str += "→ childs[";
				str += idx;
				str += "] = ";
			}
			str += object.name;
			if( object == this.currentElement ) str +=  "*";
			if( object.prevFocus ) str += " (prev:" + object.prevFocus.name + ") ";
			str += "\n";
		
		tab += "    ";

		for( let i = 0; i < object.childs.length; i++ ) {
			let child = object.childs[ i ];
			str += this.debug_dir( child, tab, i );
		}
		return str;
	}
}//App
//---▲▲▲ 3 App ▲▲▲
//---
//---◆◆◆ 4 MessageWindow ◆◆◆
class MessageWindow extends Element {//c
	constructor( style, app ) {
		//check.　デフォルト値と上書き
		style = Object.assign( {
			gridCols : app.style.gridCols - 2,
			gridRows : app.style.gridRows - 2,
		}, style );
		//下記で自身のgridCols,gridRowsを参照しているので、２つに分けて上記であらかじめ定義
		style = Object.assign( {
			gridCol : 1,
			gridRow : 1,
			innerCols : style.gridCols / app.style.gridsPerInnerCell,
			innerRows : style.gridRows / app.style.gridsPerInnerCell,
		}, style );

		super( style, app );

		this.clear();
		
		//フラグ
		this.endedByOutside = false;
	}
	close() {//m
		this.parent.removeChild( this );
	}
	clear() {//m
		this.inner = new Array();
			for( let r = 0; r < this.style.innerRows; r++ ) {
				this.inner.push( new Array() );
			}
		this.innerCursorCol = 0;
		this.innerCursorRow = 0;
	}
	write( message, flgs ) {//m
			//check.
			if( ! flgs ) {
				
				flgs = {
					clear : false,	//メッセージ表示前文字を全消去したいときtrue
					rapid : false,	//メッセージを一気に表示したいときtrue
					wait : false,	//メッセージ表示後キー入力待ちしたいときtrue
				}
			}
			//check.　"外部により終了させられた"フラグ（１文字ずつ表示を一気に等）
			this.endedByOutside = false;
				
		this.message = message;
		this.flgs = flgs;
		this.seek = 0;
		this.isWaitingKeyNow = false;

			//check.
			if( flgs.rapid ) {
				while( this.writeChar() );
				return;
			}
			
		return new Promise( function( promiseOk ) {
			console.log( "promised 'message typing'" );
			this.promiseOk = promiseOk;
			this.metronomeForWrite = this.app.addMetronome( "writeCharTiming", 100 );
			this.metronomeForCursor = this.app.addMetronome( "messageCursor", 400 );
			
			//１文字ずつ表示するアニメーション
			this.app.anms.push( function() {
				
					//check.　メトロノームに合わせる
					if( ! this.metronomeForWrite.changed ) return;
				
				//終了チェック兼１文字出力
				if( this.endedByOutside || ! this.writeChar() ) {
						//check.　キー待ちしない
						if( ! this.flgs.wait ) {
							console.log( "promiseOk 'message typing'" );
							this.promiseOk();
						} else {
							this.isWaitingKeyNow = true;
							//キー待ちする場合は、typeKey()のほうでthis.promiseOk()される
						}

					return true;	//アニメ終了の意
				}
			}.bind( this ) );//anm
		}.bind( this ) ).then( function() {
			//promiseOk()されたらここに来る
			console.log( "promise 'message typing' completed" );
			this.app.removeMetronome( this.metronomeForWrite );
			this.app.removeMetronome( this.metronomeForCursor );
		}.bind( this ) );//new Promise
	}//write()
	// |
	// V
	writeChar() {//m
		/*	memo.
			messageの末尾時にfalseを返す
		*/
		let ch = this.message.substr( this.seek, 1 );
		let kaigyo = false;
		
		switch( ch ) {
			case "\n":
				kaigyo = true;
				break;
			default:
				this.inner[ this.innerCursorRow ][ this.innerCursorCol ] = ch;
				this.innerCursorCol ++;
					//check.　右端
					if( this.innerCursorCol == this.style.innerCols ) kaigyo = true;
		}//switch
		
			//check.　改行
			if( kaigyo ) {
				this.innerCursorCol = 0;
				//ウィンドウの途中行
				if( this.innerCursorRow < this.style.innerRows - 1 ) {
					this.innerCursorRow ++;
				} else {
					//ウィンドウの最下行
					//１行スクロール
					this.inner.shift();
					this.inner.push( new Array() );
				}
			}
		
		this.seek ++;
		
		return this.seek < this.message.length;	//messageの末尾時false

	}//writeChar()
	typeKey( key ) {
		if( this.seek < this.message.length ) {
			//文字出力中のときは文字出力を一気に行う
			for( let i = this.seek; i < this.message.length; i++ ) {
				this.writeChar();
			}
			this.endedByOutside = true;
		} else if( this.flgs.wait ) {
			//文字出力終了のときは、呼び出し元に処理終了を知らせる。
			this.promiseOk();
		}
	}//typeKey()
	draw( cc ) {//m
		super.draw( cc );
		cc.save();	
		//	cc.translate( this.x, this.y );
			
			//メッセージを描画
			cc.font = this.style.innerCellSize + "px ''";
			cc.fillStyle = "black";
			for( let r = 0; r < this.style.innerRows; r++ ) {
				let gy = this.style.innerCellSize * r;
				for( let c = 0; c < this.style.innerCols; c++ ) {
					let gx = this.style.innerCellSize * c;
					let ch = this.inner[ r ][ c ];
						//check.　行末
						if( ! ch ) break;
					cc.fillText( ch, gx, gy + this.style.innerCellSize );
				}
			}
			
			//キー待ちを示すカーソルを描画
			if( this.isWaitingKeyNow && this.metronomeForCursor.toggle ) {
				let gx = this.innerCursorCol * this.style.innerCellSize;
				let gy = this.innerCursorRow * this.style.innerCellSize;
				cc.fillText( "▼", gx, gy + this.style.innerCellSize );
			}
			
		cc.restore();
	}//draw()
}//MessageWindow
// |
// V
//---■■■ 5 List ■■■
class List extends Element {//c
	constructor( src, style, app ) {
			//check.　styleのデフォルト値と上書き
			style = Object.assign( {
				innerCellSize : app.style.gridCellSize,
				gridCol : 0,
				gridRow : 0,
			}, style );
			
		//リストの項目
		let items;
			if( src.items instanceof Function )
				items = src.items.call( app );	//関数から得る
			else
				items = src.items;
			
			//check.　リストの列数
			if( style.gridCols == undefined ) {
				//各列において、その全行の文字列の中での最大文字数を得る
				//その最大文字数をstyle.gridColsへ加算していき、ウィンドウの横幅とする
				style.gridCols = 0;
				for( let column of src.columns ) {
					let strings = new Array();
						for( let item of items ) {
							let string = ( item[ column.key ] instanceof Function )
											? item[ column.key ].call( app )
											: item[ column.key ];
							strings.push( string );
						}
					column.cols = Utl.maxLengthBinDiv2( strings ) + 1;	//その列の幅
					column.col = style.gridCols;						//その列の位置
					style.gridCols += column.cols;
				}
					//check.　タイトルのほうが長ければ
					if( src.title != undefined ) {
						style.gridCols = Math.max( Utl.lengthBinDiv2( src.title ), style.gridCols );
					}
					//check.
					if( style.gridPaddingLeft != undefined ) style.gridCols += style.gridPaddingLeft;
			}
			//check.　リストの行数
			if( style.gridRows == undefined ) {
				style.gridRows = items.length;
					//check.
					if( src.title ) style.gridRows++;
			}

		super( style, app );
		
		this.src = src;
		this.items = items;
	}
	draw( cc ) {//m

		this.drawWindow( cc );
		
			//debug.　テキストエリアに水色枠
			if( 0 ) {
				cc.strokeStyle = "cyan";
				cc.strokeRect( 0, 0, this.w, this.h );
			}
			
		cc.fillStyle = "black";
		cc.font = this.style.innerCellSize + "px ''";

		//タイトル
		if( this.src.title ) {
			let gx = ( this.style.gridPaddingLeft - 0.3 ) * this.app.style.gridCellSize;
			let gy = this.style.innerCellSize * 0.65;
			cc.fillText( this.src.title, gx, gy );
		}
		
		//リスト
		cc.save();
			this.translateToItemArea( cc );
			
			//行
			for( let i = 0; i < this.items.length; i++ ) {
				let item = this.items[ i ];
				let gy = this.style.innerCellSize * i;
				//列
				for( let col = 0; col < this.src.columns.length; col++ ) {
					let column = this.src.columns[ col ];
					//値が関数になっている場合に対応　たとえば、shopの「所持金」表示
					let string = ( item[ column.key ] instanceof Function )
								? item[ column.key ].call( this.app )
								: item[ column.key ];
					let gx = column.col * this.style.innerCellSize;
						//check.　右寄せ指定　たとえば数値表示
						if( Utl.accessBy( column, "align" ) == "right" ) {
							gx += column.cols * this.style.innerCellSize;
							gx -= cc.measureText( string ).width;
						}
						
					cc.fillText( string, gx, gy + this.style.innerCellSize );
						//debug.　列に緑枠
						if( 0 ) {
							cc.strokeStyle = "green";
							cc.strokeRect(
								column.col * this.style.innerCellSize,
								gy,
								column.cols * this.style.innerCellSize,
								this.style.innerCellSize
							);
						}
				}//for
			}//for
		cc.restore();
	}//draw()
	translateToItemArea( cc ) {//m
		if( this.src.title ) cc.translate( 0, this.style.innerCellSize );
		cc.translate( this.style.gridPaddingLeft * this.style.innerCellSize, 0 );
	}
}//List
// |
// V
//---◆◆◆ 6 Menu ◆◆◆
class Menu extends List {//c
	constructor( mode, context, src, style, app ) {
			//check.　表の列設定が未定義なら１列でnameを並べるだけの通常メニュー
			if( src.columns == undefined ) {
				src.columns = [ { key : "name" } ];
			}
			//check.　左端にカーソル用の空白エリア
			if( style.gridPaddingLeft == undefined ) style.gridPaddingLeft = 1;

		super( src, style, app );
		
		this.mode = mode;
		this.context = context;
		this.src = src;
			//check.　contextをthisで参照できるように
			if( this.src.context ) this.context = this.src.context;
		
		//カーソルが横にも動く場合のため（未対応中）
		this.itemCols = 1;					//現在使用していない。
		this.itemRows = this.items.length;	//縦方向は基本なのでこれは使用している
		
		this.cursorX = 0;
		this.cursorY = 0;
	}
	select() {//m
		return new Promise(
			function( promiseOk ) {
				console.log( "promised 'select'" );
				this.promiseOk = promiseOk;
			}.bind( this )
		).then(
			function( result ) {
				console.log( "promise 'select' completed" )
				return result;
			}
		);
	}
	typeKey( key ) {//m
		let addX = ( key == 39 ) - ( key == 37 );
		let addY = ( key == 40 ) - ( key == 38 );
		
		if( addX || addY ) {
			this.cursorX += addX;
			this.cursorY += addY;
				//check.
				if( this.cursorY < 0 ) this.cursorY = 0;
				if( this.cursorY >= this.itemRows ) this.cursorY = this.itemRows - 1;
				
			//カーソル　移動のタイミングで「点滅の点灯開始状態」にする
			//こうしないとチラチラして見づらい
			this.app.metronomez.forCursor.toggle = true;
			this.app.metronomez.forCursor.time = 0;
		} else {
			
			switch( key ) {
				case 32://SPACE
					this.app.artz.soundz.pushA.rapidPlay();

					this.selectedItem = this.items[ this.cursorY ];

					//選択した項目の形態
					let menusrc;
						//選択した項目はメニューであるA
						if( this.selectedItem.items ) {
							menusrc = this.selectedItem;
						} else if( this.src.assistMenu ) {
							//選択した項目はメニューではないが、
							//this.srcにassistMenu指定がある。B
							menusrc = Utl.copyObjectKai( this.src.assistMenu );
							menusrc.items = this.selectedItem[ this.src.name ];
						} else if( this.selectedItem[ "assistMenu" + this.context ] ) {
							//選択した項目はメニューではないが、
							//this.selectedItem内にcontextMenu指定がある。C
							menusrc = this.selectedItem[ "assistMenu" + this.context ];
						} else {
							//メニューの終了
							//ストーリー内でメニューの結果を待っている場合
							if( this.mode == "selectmode" ) {
								//選択結果を返す
								this.promiseOk( this.getResult() );
							} else {
								//campなどでオペレーションする場合
								//埋め込まれたスクリプトをもとに実行する
								this.execute();
							}
							return;
						}
					
					//次のメニューを表示
					let col = menusrc.col == undefined
								? this.selectedItem.name.length + 2
								: menusrc.col;
					let row = menusrc.row == undefined
								? this.cursorY + 2
								: menusrc.row;
					let menu = new Menu( this.mode, this.context, menusrc, {
						gridCol : col,
						gridRow : row,
					}, this.app );
					this.appendChild( menu );
					menu.promiseOk = this.promiseOk;
					
					break;
				case 66://B
					//セレクトモードでトップメニューがキャンセルされた
					let thisIsTop = ! ( this.parent instanceof Menu );
					if( this.mode == "selectmode" && thisIsTop ) {
						this.promiseOk( null );
					} else {
						//それ以外
						this.parent.removeChild( this );
					}
					break;
			}//switch
		}//if else
	}//typeKey()
	// |
	// V
	getResult() {//m
		let result = new Object();
			result.menus = new Array();
				//自分自身から上へさかのぼってmenusへ加えていく
				let element = this;
				while( element instanceof Menu ) {
					result.menus.unshift( element );
					element = element.parent;
				}
			result.argz = new Object();
			for( let menu of result.menus ) {
				if( menu.src.story ) result.story = menu.src.story;		//埋め込みスクリプト
				result.argz[ menu.src.argzName ] = menu.selectedItem;	//そのための引数
			}
			result.selectedItem = result.menus[ result.menus.length - 1 ].selectedItem;

			/*
			メニューの選択結果は、
				result.menus		たどったメニュー
				result.argz			たどって得られた引数リスト
				result.story		たどりの途中で得られた埋め込みスクリプト
				result.selectedItem	最後に選択した項目
			なお、storyとargzは
				story( argz )
				のように実行する。
				argzはstoryの中で、argz.char1などのように参照する。
				storyはこのファイルの最初のほうのメニュー定義部分などで定義されているもの。
				argz.char1のchar1などは、同じくメニュー定義部分で、argzKey : "char1"
				のように定義されている。
				storyの内容も、storyの中でどんな引数を使うかもユーザーが定義する。
			*/
		return result;
	}
	async execute() {//m
		let result = this.getResult();

			//check.
			if( ! result.story ) {
				alert( "storyが未定義の状態です。at menu.execute()" );
			}
			
		await result.story.call( this.app, result.argz );
 	
			//check.　storyによってサブメニューの元となったモノがなくなった。
			/*
			たとえば、
				コマンド？：どうぐ
					だれの？：キャラ１
						どれ？：なんこう
							どうする？：つかう
								だれに？：キャラ１
			として、なんこうを使ったら、なんこうはなくなるので、
			なんこうに由来する　どうする？　だれに？　の２つのメニューは
			消さなくてはならない。その処理↓
			*/
			for( let i = result.menus.length - 1; i > 0; i -- ) {
				let menu = result.menus[ i ];
				let selfItem = menu.parent.selectedItem;
				//親メニューに自身の選択項目がない。
				if( ! menu.parent.items.includes( selfItem ) ) {
					menu.parent.removeChild( menu );
					//メニューを１つ消せば以降のサブメニューはchildなので一緒に消える
				}
			}
	}
	draw( cc ) {//m
		super.draw( cc );	//extends List
			
		cc.save();
			
			this.translateToItemArea( cc );

			//カーソルを描画
			if( this.app.currentElement != this ||
				this.app.metronomez.forCursor.toggle ) {
				let w = this.style.innerCellSize * .7;
				let h = this.style.innerCellSize * .8;
				let gx = - this.style.innerCellSize + ( this.style.innerCellSize - w ) / 2;
				let gy = this.cursorY * this.style.innerCellSize + ( this.style.innerCellSize - h ) / 2 + 1;
				cc.beginPath();
				cc.moveTo( gx, gy );
				cc.lineTo( gx + w, gy + h / 2 );
				cc.lineTo( gx, gy + h );
				cc.closePath();
				cc.fillStyle = "black";
				cc.fill();
			}
		
		cc.restore();
		
		super.drawChildren( cc );
	}//draw()
}//Menu
//---■■■ 7 Title ■■■
class TitleScreen extends Element {//c
	constructor( style, app ) {
		super( style, app );
	}
	async typeKey( key ) {//m
		this.app.artz.soundz.pushA.play();

		this.app.removeChild( this );
		this.app.mapScreen = new MapScreen( {
			innerCellSize : this.app.style.gridCellSize * 2,
			gridCol : 0,
			gridRow : 0,
			gridCols : this.app.style.gridCols + ( this.app.style.gridCols % 2 ? 0 : 1 ),
			gridRows : this.app.style.gridRows + ( this.app.style.gridRows % 2 ? 0 : 1 ),
					//偶数なら奇数にする
		}, this.app );

		this.app.appendChild( this.app.mapScreen );
		
		let map = this.app.memoryCard.player.position.map;
		let x = this.app.memoryCard.player.position.x;
		let y = this.app.memoryCard.player.position.y;
		switch( 2 ) {
			case 0: this.app.mapScreen.playerMapMoveTo( map, x, y );	//memorycard
				break;
			case 1: this.app.mapScreen.playerMapMoveTo( map, 3, 8 );	//外
				break;
			case 2: this.app.mapScreen.playerMapMoveTo( App.townmap, 3, 5 );//町
				break;
		}
		
		this.app.openMessage();
		await this.app.writeMessage( "スタート", { wait : true } );
		this.app.closeMessage();
	}
	draw( cc ) {//m
		cc.fillStyle = "black";
		cc.fillRect( 0, 0, this.w, this.h );
		cc.font = this.style.innerCellSize + "px''";
		cc.fillStyle = "white";
		cc.fillText( "hit any key", this.w * .35, this.h * .75 );
	}
}//TitleScreen
// |
// V
//---◆◆◆ 8 Map ◆◆◆
class MapScreen extends Element {//c
	constructor( style, app ) {	
//		style = Object.assign( {
//			gridCols : app.style.gridCols,
//			gridRows : app.style.gridRows,
//		}, style );
		style = Object.assign( {
			innerCols : style.gridCols / app.style.gridsPerInnerCell,
			innerRows : style.gridRows / app.style.gridsPerInnerCell,
		}, style );

		super( style, app );

		this.actions = new Array();
		
		this.scroll = {
			count : 0,
			isAnimating : false,
			tweakGx : 0,
			tweakGy : 0,
		}
		this.keyLock = false;
	}
	assignStyle( newStyle ) {
		super.assignStyle( newStyle );

		this.style.innerColsHalf = Math.floor( this.style.innerCols / 2 );
		this.style.innerRowsHalf = Math.floor( this.style.innerRows / 2 );
	}
	// |
	// V
	tweakColForLooping( targetCol ) {//m
		//tweak.　ループ時は範囲外の値を範囲内に調整する
		if( this.cfg.maploop ) {
			if( targetCol < 0 )
				targetCol += this.mapCols;
			else if( targetCol >= this.mapCols )
				targetCol -= this.mapCols;
		}
		return targetCol;
	}
	tweakRowForLooping( targetRow ) {//m
		//tweak.　ループ時は範囲外の値を範囲内に調整する
		if( this.cfg.maploop ) {
			if( targetRow < 0 )
				targetRow += this.mapRows;
			else if( targetRow >= this.mapRows )
				targetRow -= this.mapRows;
		}
		return targetRow;
	}
	getPositionAwayBy( step ) {//m
		let dir = this.app.memoryCard.player.direction;
		let x = this.playerCol + ( ( dir == 2 ) - ( dir == 0 ) ) * step;
		let y = this.playerRow + ( ( dir == 3 ) - ( dir == 1 ) ) * step;
		return { x : x, y : y }
	}
	// |
	// V
	async typeKey( key ) {//m
		if( key == 32 ) {
				//check. 向いている方向に何かがある
				let pos = this.getPositionAwayBy( 1 );
				let bit;
				if( bit = Utl.accessBy( this.mapBits, pos.y, pos.x ) ) {
					if( bit == "□" ) {
						pos = this.getPositionAwayBy( 2 );
						this.actions.push( { x : pos.x, y : pos.y, type : "contact" } );
						return;
					}
				}
				if( bit = Utl.accessBy( this.storyBits, pos.y, pos.x ) ) {
					if( bit.sprite ) {
						this.actions.push( { x : pos.x, y : pos.y, type : "contact" } );
						return;
					}
				}

			//コマンドメニューを開く
			this.app.artz.soundz.pushA.rapidPlay();
			let campMenu = new Menu( "executemode", "Camp", this.app.menusrcz.camp, {
				gridCol : 1,
				gridRow : 1,
			}, this.app );
			this.app.appendChild( campMenu );
			campMenu.focus();
		}
	}
	// |
	// V
	senseKey( key ) {//m
			//check.
			if( this.scroll.isAnimating || this.keyLock ) return;
			
		let addX, addY;

			switch( key ) {
				case 37: this.app.memoryCard.player.direction = 0; addX = -1; addY = 0;	break;
				case 38: this.app.memoryCard.player.direction = 1; addX = 0; addY = -1;	break;
				case 39: this.app.memoryCard.player.direction = 2; addX = 1; addY = 0;	break;
				case 40: this.app.memoryCard.player.direction = 3; addX = 0; addY = 1;	break;
				default: return;
			}
			
			//移動先
			let destinCol = this.tweakColForLooping( this.playerCol + addX );
			let destinRow = this.tweakRowForLooping( this.playerRow + addY );
				//check.　移動先は壁
				let bit;
				if( bit = Utl.accessBy( this.mapBits, destinRow, destinCol ) )
					if( this.cfg.walls.indexOf( bit ) > -1 ) {
						this.app.artz.soundz.pushWall.play();
						return;
					}
				//check.　移動先はスプライト
				if( Utl.accessBy( this.storyBits, destinRow, destinCol, "sprite"  ) ) return;

		//１マススクロールの起動
		this.scroll.addX = addX;
		this.scroll.addY = addY;
		this.scroll.count = 0;
		this.scroll.isAnimating = true;
		
	}//senseKey
	// |
	// V
	frameForScroll() {//m
		//１マススクロールにおける、１ドット分のスクロール処理
		
		this.scroll.tweakGx -= this.scroll.addX;
		this.scroll.tweakGy -= this.scroll.addY;
			//tweakGx,Gyが1ドットスクロールの主要のしくみ
			//それについてはthis.draw()を参照
		
		this.scroll.count ++;
			//check.　スクロールの終了
			if( this.scroll.count == this.style.innerCellSize ) {
				this.scroll.isAnimating = false;
				
				this.scroll.tweakGx = 0;
				this.scroll.tweakGy = 0;
					//draw()はスクロール終了後も参照するからここで０にする。

				this.playerCol = this.tweakColForLooping( this.playerCol + this.scroll.addX );
				this.playerRow = this.tweakRowForLooping( this.playerRow + this.scroll.addY );
				
				//マップ外に乗ったストーリー発生
				if( ! this.cfg.maploop && (
						this.playerCol < 0 || this.playerCol >= this.mapCols ||
						this.playerRow < 0 || this.playerRow >= this.mapRows ) ) {
					this.actions.push( { type : "exit" } );
				} else {
					//座標に乗ったストーリー発生
					this.actions.push( { x : this.playerCol, y : this.playerRow, type : "walk" } );
				}
			}//if　スクロール終了
	}//frameForScroll()
	// |
	// V
	playerMapMoveTo( mapdatafunc /*other args*/ ) {//m
	
		//マップ間移動
			
			//check.　
			if( this.app.bgm ) {
				this.app.bgm.pause();
				this.app.bgm.currentTime = 0;
			}
		
		//データを記述した関数からデータを取り出す１
		let commentData = mapdatafunc.toString().match( /\/\*\r\n([\s\S]+)\r\n\*\// )[ 1 ];
		
		//データを記述した関数からデータを取り出す２
		this.cfg = mapdatafunc();
		
		
		//取り出したデータを使えるように加工
		this.mapBits = new Array();
		this.storyBits = new Array();	//位置に設定されたストーリー
		
			//マップデータを読み取る　ここから
			let shortcutPositionz = new Object();
			let lines = commentData.split( /\r\n/ );

			for( let y = 0; y < lines.length; y++ ) {
				let line = lines[ y ];
				
				this.mapBits[ y ] = new Array();
				for( let x = 0; x < line.length; x++ ) {
					let mapBit = line.substr( x, 1 );
					
						//check.　半角文字のときはストーリーへのショートカット
						if( mapBit.match( /[0-9a-z ]/i ) ) {
							let shortcutId = mapBit + line.substr( x + 1, 1 );
								shortcutId = shortcutId.replace( / /, "" );

								//check.　ショートカットがリンク切れ
								if( ! this.cfg.shortcuts[ shortcutId ] ) {
									alert( "マップ上に配置したショートカットがリンク切れ id:" + shortcutId );
									continue;
								}
							
							//マップ上のショートカットについては後で処理
							shortcutPositionz[ shortcutId ] = {
								x : x,
								y : y,
							};
							//ショートカットをマップデータに置き換え
							mapBit = this.cfg.shortcuts[ shortcutId ].bit;

							x++;	//1文字多く読んだから
						}
						
					this.mapBits[ y ].push( mapBit );
				}
			}//for

			this.mapCols = this.mapBits[ 0 ].length;
			this.mapRows = this.mapBits.length;
			
			//storyBits初期化
			for( let y = 0; y < this.mapRows; y++ ) {
				this.storyBits[ y ] = new Array();
				for( let x = 0; x < this.mapCols; x++ ) {
					this.storyBits[ y ][ x ] = null;
				}
			}
			//storyBitsにstoryを配置
			for( let shortcutId in shortcutPositionz ) {
				let pos =shortcutPositionz[ shortcutId ]; 
				let shortcut = this.cfg.shortcuts[ shortcutId ];
				this.storyBits[ pos.y ][ pos.x ] = shortcut;
								
					//tweak.　座標情報をセット
					shortcut.x = pos.x;
					shortcut.y = pos.y;
			}
			
			
			//マップデータの端を超えるとマップの反対側になる計算
			if( this.cfg.maploop ) {
				this.patchwork = function( position, size ) { return ( position + size ) % size }
			} else {
				//町などループしない場合
				this.patchwork = function( position ) { return position };
			}

			
			//マップデータを読み取る　ここまで


		//BGM
		if( this.cfg.bgmTitle ) {
			this.app.bgm = this.app.artz.bgms[ this.cfg.bgmTitle ];
			this.app.bgm.loop = true;
			this.app.bgm.play();
		}
				
		//メソッドの引数によって処理を分ける（オーバーロードみたいに）
		let argtypes;
			let argArray = Array.from( arguments );
			argtypes = argArray.map( arg => ( typeof arg ).substr( 0, 1 ) ).join( "" );
		
		//ショートカットを指定して移動
		if( argtypes.indexOf( "fs" ) == 0 ) {
		
			//playerMapMoveTo( function, string [,number] )
			let shortcut = arguments[ 1 ];
			this.playerCol = this.cfg.shortcuts[ shortcut ].x;
			this.playerRow = this.cfg.shortcuts[ shortcut ].y;
				//check.
				if( typeof arguments[ 2 ] !== "undefined" )
					this.app.memoryCard.player.direction = arguments[ 2 ];
					
		} else if( argtypes.indexOf( "fnn" ) == 0 ) {
		
			//移動先位置の座標指定して移動
			//playerMapMoveTo( function, number, number [,number] )
			this.playerCol = arguments[ 1 ];
			this.playerRow = arguments[ 2 ];
				//check.
				if( typeof arguments[ 3 ] !== "undefined" )
					this.app.memoryCard.player.direction = arguments[ 3 ];
		} else {
		
			alert( "playerMapMoveTo()の引数リストが想定外 : " + argtypes );
		
		}
		
	}//playerMapMoveTo()
	// |
	// V
	playerMapMoveTo_fx( mapdatafunc /*other args*/ ) {//m
		
		//視覚効果付きのマップ間移動
	
		let argArray = Array.from( arguments );
			//check.　歩く音
			let donotFootsteps = false;
			if( ( typeof argArray[ argArray.length - 1 ] ) == "boolean" ) {
				donotFootsteps = argArray.pop();
			}
			if( ! donotFootsteps ) this.app.artz.soundz.walk.play();

		this.keyLock = true;

		//クロージャ（関数が終わっても、この関数内で定義した関数内では生き続ける変数）
		let countMax = 3;
		let count = countMax;
		let flg = true;
		//マップ間移動時の画面フェードアウト、イン
		let metronome = this.app.addMetronome( "forFade", 125 );
		this.app.anms.push( function() {
				//check.　メトロノームに合わせて
				if( ! metronome.changed ) return;
			
			if( flg ) {
				//フェードアウト
				count --;
				let value = 100 * count / countMax;
				this.app.cc.canvas.style.filter = "brightness( " + value + "% ) ";
					//check.	フェードアウトの終了
					if( ! count ) {
						//マップ間移動する
						this.playerMapMoveTo( ...argArray );
						flg = false;
						count = countMax;
					}
			} else {
				//フェードイン
				count --;
				let value = 100 * ( countMax - count ) / countMax;
				this.app.cc.canvas.style.filter = "brightness( " + value + "% ) ";
					//check.	フェードインの終了（全終了）
					if( ! count ) {
						this.keyLock = false;
						this.app.removeMetronome( metronome );
						return true;	//trueはアニメの終了の意
					}
			}
		}.bind( this ) );
	}//playerMapMoveTo_fx()

	searchStory() {//m
		let hits = new Array();
		
		/*	優先順位
		処理順		1.		2.		3.
		action		walk＞walk　　＞walk
		x,y			全体＞ある位置＞全体
		例			沼地＞ジャンプ＞エンカウント
		
		つまり、一歩進んでwalkアクションになると、３種類のストーリーが起こりうる。
		たとえば、沼地に入ったら沼地のダメージのストーリーがまず行われ、
		つづいて、マップジャンプがそこに設定されていたらジャンプのストーリーが行われる。
		最後にエンカウントのストーリーが入るが、ジャンプのストーリーは
		打ち切りのフラグをreturnするので、エンカウントは行われない。
		優先順位を考慮しないと、マップジャンプしたあとに突然戦闘になるとか、
		沼地に踏み込んだのに、ダメージが減らないままマップジャンプするとか
		になってしまう。
		*/
		
		for( let action of this.actions ) {
			let story;

			//1. マップチップのストーリー検索
			let bit;
				if( action.y != undefined && action.x != undefined )
					bit = this.mapBits[ action.y ][ action.x ];
				else
					bit = this.cfg.outside;
			if( this.cfg.mapBitStoriez[ bit ] )
			if( story = this.cfg.mapBitStoriez[ bit ][ action.type ] ) {
				hits.push( { action : action, story : story } );
			}

			//2. マップ上の位置に設定されたストーリー
			if( story = Utl.accessBy( this.storyBits, action.y, action.x, "story", action.type ) ) {
				hits.push( { action : action, story : story } );
			}
			
			//3. マップ自体に設定されたストーリー
			if( story = this.cfg.story[ action.type ] ) {
				hits.push( { action : action, story : story } );
			}
		}
		
		this.actions.length = 0;
		return hits;
	}
	draw( cc ) {//m
		cc.fillStyle = "black";
		cc.fillRect( this.x, this.y, this.w, this.h );

		let image;
		
		cc.save();
			cc.translate(
				-this.style.innerCellSize / 2 + this.scroll.tweakGx,
				-this.style.innerCellSize / 2 + this.scroll.tweakGy
			);
				//tweakX,Yが描画位置を1ドットずつずらすので
				//きれいに1ドットスクロールしてるようにみえる
			
			cc.font = this.style.innerCellSize + "px ''";
			cc.fillStyle = "rgb(0,255,0)";
			
			//this.playerCol,this.playerRowが画面中央に来るように描画開始座標を調整
			let sx = this.playerCol - this.style.innerColsHalf;
			let sy = this.playerRow - this.style.innerRowsHalf;
			
			for( let inner_row = 0; inner_row < this.style.innerRows; inner_row++ ) {
				let map_row = this.patchwork( sy + inner_row, this.mapRows );//両端ループ処理
				let gy = inner_row * this.style.innerCellSize;
				for( let inner_col = 0; inner_col < this.style.innerCols; inner_col++ ) {
					let map_col = this.patchwork( sx + inner_col, this.mapCols );//両端ループ処理
					let gx = inner_col * this.style.innerCellSize;
					
					//マップを描画
					let bit = Utl.accessBy( this.mapBits, map_row, map_col );
						//check.
						if( bit == undefined ) bit = this.cfg.outside;
					image = this.cfg.images[ bit ] ? this.cfg.images[ bit ] : bit;
					cc.fillText( image, gx, gy + this.style.innerCellSize );

					//スプライトを描画
					let story = Utl.accessBy( this.storyBits, map_row, map_col );
						//check.
						if( ! story ) continue;
						if( ! story.sprite ) continue;
					image = story.sprite;
					cc.fillText( image, gx, gy + this.style.innerCellSize );
				}
			}


			//debug.　グリッド
			if( 0 ) {
				cc.strokeStyle = "rgba(255,255,255,.125)";
				for( let row = 0; row < this.style.innerRows; row ++ ) {
					for( let col = 0; col < this.style.innerCols; col ++ ) {
						let x = col * this.style.innerCellSize;
						let y = row * this.style.innerCellSize;
						cc.strokeRect( x, y, this.style.innerCellSize, this.style.innerCellSize );
					}
				}
			}

		cc.restore();

			//debug.　画面の十字線
			if( 0 ) {
				cc.strokeStyle = "red";
				cc.beginPath();
				cc.moveTo( 0, cc.canvasHeight / 2 );
				cc.lineTo( cc.canvasWidth, cc.canvasHeight / 2 );
				cc.stroke();
				cc.beginPath();
				cc.moveTo( cc.canvasWidth / 2, 0 );
				cc.lineTo( cc.canvasWidth / 2, cc.canvasHeight );
				cc.stroke();
			}
		
		//プレイヤーキャラ描画
		cc.fillStyle = "cyan";
		cc.font = this.style.innerCellSize + "px ''";
		let gx = ( cc.canvasWidth - this.style.innerCellSize ) / 2;
		let gy = ( cc.canvasHeight - this.style.innerCellSize ) / 2 + this.style.innerCellSize;
		let sprite = this.app.artz.spritez.player;
		let idx = this.app.memoryCard.player.direction * 2 + sprite.metronome.index;
		image = sprite.images[ idx ];
		cc.fillText( image, gx, gy );
			//debug.　座標表示
			if( 0 ) {
				cc.fillStyle = "red";
				cc.fillText( this.playerCol + "," + this.playerRow, gx, gy );
			}


		this.drawChildren( cc );
		
	}//draw()
}//MapScreen

//---■■■ 9 BattleView ■■■
class BattleView extends Element {//c
	constructor( monsters, style, app ) {
		super( style, app );
		this.monsters = monsters;
	}
	draw( cc ) {//m
		cc.drawImage( this.app.artz.imagez.battleBG, 0, 0, this.w, this.h );
		if( 1 ) {
			let step = this.w / ( this.monsters.length );
			for( let i = 0; i < this.monsters.length; i++ ) {
				let monster = this.monsters[ i ];
					//check.　倒れたモンスター
					if( monster.property.hp == 0 ) continue;
				let zm = 0.1;
				let mw = monster.image.width * zm;
				let mh = monster.image.height * zm;
				let gx = i * step + this.w / this.monsters.length / 2 - mw / 2;
				let gy = this.h * 0.9 - mh;
				cc.drawImage( monster.image, gx, gy, mw, mh );
				
			}
		} else {
			let step = this.w / ( this.monsters.length + 1 );
			for( let i = 0; i < this.monsters.length; i++ ) {
				let monster = this.monsters[ i ];
					//check.　倒れたモンスター
					if( monster.property.hp == 0 ) continue;
				let zm = 0.1;
				let mw = monster.image.width * zm;
				let mh = monster.image.height * zm;
				let gx = ( i + 1 ) * step - mw / 2;
				let gy = this.h * 0.9 - mh;
				cc.drawImage( monster.image, gx, gy, mw, mh );
				
			}
		}
	}
}


App.worldmap = function() {/*
●〃〃〃〃〃〃〃～～～～～～～〃〃〃〃〃〃〃〃〃〃〃￥￥￥▲〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
￥山山山山山〃～～～～～～￥￥〃〃〃〃〃〃〃〃〃￥￥山山山￥〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
￥山山山〃〃〃〃～～～〃〃〃〃〃〃〃〃〃〃〃〃￥￥山山山￥￥〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
￥山山〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃￥￥山山山￥￥〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
￥￥〃〃1 〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃￥￥山山山￥￥〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃￥￥山山山￥￥〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
〃沼沼沼〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃￥￥￥山山山￥￥〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
〃沼沼沼〃〃〃〃〃山山〃〃〃〃〃〃〃￥￥￥山山山￥￥〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
〃沼沼沼〃〃〃山山山〃〃〃〃〃〃〃￥￥￥山山山￥￥〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
〃〃〃〃山山山山山〃〃〃〃〃〃〃〃￥￥山山山山〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
〃〃〃山山山〃〃〃〃〃〃〃〃￥〃〃￥山山山山山〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
〃〃〃〃〃〃〃〃〃〃〃〃〃￥￥￥〃〃〃〃山山山山山山〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
〃〃〃〃〃〃〃〃〃〃〃〃〃〃￥〃〃〃〃〃山山山山山山山〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
山山山〃〃〃～～～〃〃〃〃〃〃〃〃〃〃山山〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
山山〃〃～～～～￥￥〃〃〃〃〃〃〃〃山山山山〃〃〃〃〃〃〃山〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
山〃〃～～～￥￥￥〃〃〃〃〃〃〃〃山山山山山山山〃〃〃〃〃山〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
〃〃〃〃￥￥￥￥〃〃〃〃〃〃〃〃〃山山山山山山山山〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
〃〃〃〃〃〃￥￥￥〃〃〃〃〃〃〃山山山山山山山山山山～～〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
〃〃〃〃〃〃〃￥￥￥〃〃〃〃〃〃山山山山山～～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
〃〃〃〃〃〃〃￥￥￥〃〃〃〃〃〃〃〃〃〃～～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
〃〃〃〃￥￥￥￥￥￥￥〃〃〃〃〃〃〃〃～～～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
〃〃〃￥￥￥￥￥￥￥￥〃〃〃〃〃〃〃〃～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
〃〃￥￥￥￥￥￥〃〃〃〃〃〃〃〃〃～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
〃￥￥￥￥￥〃〃〃〃〃〃〃〃〃〃～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
〃〃￥￥〃〃〃〃〃〃〃〃〃〃～～～～～〃〃〃〃〃〃〃￥￥〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
〃〃〃〃〃〃〃〃〃〃〃〃～～～～～～～〃〃〃〃〃〃￥￥〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
〃〃〃〃〃〃〃〃〃〃〃～～～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
○〃〃〃〃〃〃〃〃～～～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃△〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
○〃〃〃〃〃〃〃〃～～～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃△〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
○〃〃〃〃〃〃〃〃～～～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃△〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
○〃〃〃〃〃〃〃〃～～～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃△〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
○〃〃〃〃〃〃〃〃～～～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃△〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
○〃〃〃〃〃〃〃〃～～～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃△〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
○〃〃〃〃〃〃〃〃～～～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃△〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
○〃〃〃〃〃〃〃〃～～～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃△〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
○〃〃〃〃〃〃〃〃～～～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃△〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
○〃〃〃〃〃〃〃〃～～～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃△〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
○〃〃〃〃〃〃〃〃～～～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃△〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
○〃〃〃〃〃〃〃〃～～～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃△〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
○〃〃〃〃〃〃〃〃～～～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃△〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
○〃〃〃〃〃〃〃〃～～～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃△〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
○〃〃〃〃〃〃〃〃～～～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃△〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
○〃〃〃〃〃〃〃〃～～～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃△〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
○〃〃〃〃〃〃〃〃～～～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃△〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
○〃〃〃〃〃〃〃〃～～～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃△〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
○〃〃〃〃〃〃〃〃～～～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃△〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
○〃〃〃〃〃〃〃〃～～～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃△〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
○〃〃〃〃〃〃〃〃～～～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃△〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
○〃〃〃〃〃〃〃〃～～～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃△〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
○〃〃〃〃〃〃〃〃～～～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃△〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
○〃〃〃〃〃〃〃〃～～～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃△〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
○〃〃〃〃〃〃〃〃～～～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃△〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
○〃〃〃〃〃〃〃〃～～～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃△〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
○〃〃〃〃〃〃〃〃～～～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃△〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
○〃〃〃〃〃〃〃〃～～～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃△〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
○〃〃〃〃〃〃〃〃～～～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃△〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
○〃〃〃〃〃〃〃〃～～～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃△〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
○〃〃〃〃〃〃〃〃～～～～～～～〃〃〃〃〃〃〃〃〃〃〃〃〃△〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃山山山山
*/
	return {
		maploop : true,
		bgmTitle : "field",
		walls : "～",
		shortcuts : {
			"1" : {
				bit : "凸",
				story : {
					"walk" : function( act ) {
						this.mapScreen.playerMapMoveTo_fx( App.townmap, "s", 2 );
						return true;	//他のstory実行を打ち切る意
					},
				},
			},
		},
		story : {
			"walk" : function( act ) {
				//エンカウント
				if( Math.random() > .8 ) {
					this.encounter();
				}
			},
		},
		mapBitStoriez : {
			"沼" : {
				"walk" : async function() {
					await this.effect_flash( "darkred" );
				},
			},
		},
		images : {
			"￥" : "🌲",
			"凸" : "🏰",
			"山" : "⛰",
			"～" : "🌊",
			"沼" : "☠",
		},
	}//return
}//worldmap


App.townmap = function() {/*
木木木木木木木〃・・〃木木木木木木木木木〃・・〃〃〃〃〃〃〃
木■■■■■木〃・・〃木■■■■■■■木〃・・〃〃〃〃〃〃〃
木■刀刀刀■木〃・・〃木■■■■■■■木〃・・〃〃〃〃〃〃〃
木■・ws・■木〃・・〃木■■■■■■■木〃・・〃〃〃〃〃〃〃
木■■□■■木〃・・〃木■■■扉■■■木〃・・〃〃〃〃〃〃〃
木木〃・剣木木〃・・〃〃花花花・花花花a2〃・・〃〃〃〃〃〃〃
s ・・・・・・・・・・・・・・・・・・・・・・・・・・・・・
・・・・・・・・・・・・・・・・・・・・・・・・・・・・・・
木木盾・〃木木〃・・〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃
木■■□■■木〃・・〃a1〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃
木■・ss・■木〃・・〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃
木■盾盾盾■木〃・・〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃
木■■■■■木〃・・〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃
木木木木木木木〃・・〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃
〃〃〃〃〃〃〃〃・・〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃
〃〃〃〃〃〃〃〃・・〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃
〃〃〃〃〃沼沼〃・・〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃
〃〃〃〃沼沼沼〃・・〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃
〃〃〃沼沼沼沼〃・・〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃
〃〃〃沼沼沼沼〃・・〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃
〃〃〃〃〃沼沼沼・・〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃
〃〃〃〃〃〃〃〃・・〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃
〃〃〃〃〃〃〃〃・・〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃
〃〃〃〃〃〃〃〃・・〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃
〃〃〃〃〃〃〃〃・・〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃
〃〃〃〃〃〃〃〃・・〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃
〃〃〃〃〃〃〃〃・・〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃
〃〃〃〃〃〃〃〃・・〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃
〃〃〃〃〃〃〃〃・・〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃
〃〃〃〃〃〃〃〃・・〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃〃
*/
	return {
		maploop : false,
		bgmTitle : "town",
		shortcuts : {
			"s" : {
				bit : "・",
			},
			"ws" : {
				bit : "👨‍🔧",
				story : {
					"contact" : async function( e ) {
						this.openMessage();
						await this.writeMessage( "ここは刀剣屋だ。\n", { wait : true } );
						this.closeMessage();
						await this.shop( {
							name : "shop",
							title : "刀剣屋 'ロバートウッドテイル'",//全13, 半3
							items : [
								{ name : "どうけん", 		price : 100, },
								{ name : "せいどうけん",	price : 200, },
								{ name : "てっけん", 		price : 300, },
								{ name : "こうてつけん",	price : 400, },
								{ name : "いばらきけん",	price : 18000, },
								{ name : "ちばけん",		price : 78000, },
								{ name : "とっとりけん",	price : 10102, },
								{ name : "ふくしまけん",	price : 29400, },
							],
							columns : [
								{
									key : "name",
								},
								{
									key : "price",
									align : "right",
								}
							],
						}, {
							"何を買うんだ？" : "何を買うんだ？",
							"だな。" : "だな。",
							"だ。買うかね？" : "だ。買うかね？",
							"だ。買うかね？" : "だ。買うかね？",
							"まいどあり！" : "まいどあり！",
							"ありがとうございました！" : "ありがとうございました！",
						} );
						console.log( "shop closed." );
					},
				},
			},
			"ss" : {
				bit : "👩‍🔧",
				story : {
					"contact" : async function( e ) {
						this.openMessage();
						await this.writeMessage( "ここは道具屋よ。\n", { wait : true } );
						this.closeMessage();
						await this.shop( {
							name : "shop",
							title : "道具屋 'セリーヌ・デ・イオン'",//全13, 半3
							items : [
								{ name : "なんこう", 		price : 100, },
								{ name : "羊皮紙",			price : 200, },
								{ name : "黄金の羊",			price : 200000, },
							],
							columns : [
								{
									key : "name",
								},
								{
									key : "price",
									align : "right",
								}
							],
						}, {
							"何を買うんだ？" : "何を買いますか？",
							"だな。" : "ね。",
							"だ。買うかね？" : "です。買いますか？",
							"まいどあり！" : "ありがとうございます！",
							"ありがとうございました！" : "ありがとうございました！",
						} );
						console.log( "shop closed." );
					},
				},
			},
			"a1" : {
				bit : "〃",
				sprite : "🕵️‍♂️",
				story : {
					"contact" : async function( e ) {
						this.openMessage();
						await this.writeMessage( "ドラゴンなら北のほうへ飛んで行ったぜ", { wait : true } );
						this.closeMessage();
					},
				},
			},
			"a2" : {
				bit : "〃",
				sprite : "👩‍🚒",
				story : {
					"contact" : async function( e ) {
						this.openMessage();
						await this.writeMessage( "あなたのつるぎの使い方、\nふつうじゃないよね？", { wait : true } );
						this.closeMessage();
					},
				},
			},
		},
		story : {
			"exit" : function( e ) {
				this.mapScreen.playerMapMoveTo_fx( App.worldmap, "1", 3 );
			},
		},

		outside : "〃",
		walls : "■□扉",
		mapBitStoriez : {
			"沼" : {
				"walk" : function() {
					this.effect_flash( "darkred" );
				},
				"exit" : function() {
					this.effect_flash( "darkred" );
				},
			},
		},
		images : {
			"木" : "🌳",
			"花" : "🌷",
			"扉" : "🚪",
			"箱" : "📦",
			"座" : "🪑",
			"寝" : "🛏",
			"∪" : "🏺",
			"剣" : "⚔",
			"刀" : "🗡",
			"盾" : "🛡",
			"沼" : "☠",
		},
	}//return
}//townmap

